#!/usr/bin/python

import io
import readline
from numpy import *

def search_array(array,value):
# returns array index for value
    for index,item in enumerate(array):
       if item.strip() == value:
         return index

def atnr2res(nr,atoms):
# search array atoms for atom number nr, returns atom residue name

    atom_res=atoms[int(nr)-1].split()[3]
    return atom_res

def atnr2resNr(nr,atoms):
# search array atoms for atom number nr, returns atom residue number as integer

    atom_resnr=atoms[int(nr)-1].split()[2]
    atom_resnr=int(atom_resnr)
    return atom_resnr

def inter(pair1,pair2,atoms,array):
    
    vW6 = {
    "H":0.,
    "CH0R":6.873E-2,
    "CH1R":6.873E-2,
    "CH2R":6.873E-2, 
    "CH0":4.838E-2, 
    "CH1":5.396E-2,
    "CH2":6.873E-2, 
    "CH3":8.278E-2, 
    "HO":0.,
    "OR":4.756E-2,
    "OA":4.756E-2, 
    "OE":4.756E-2,
    "NT":4.936E-2,  # 56Acarbo_cht
    "NL":4.936E-2,	# 56Acarbo_cht
    "N":4.936E-2	# 56Acarbo_cht
    }
    vW12 = {
    "H":0.,
    "CH0R":1.36E-3,
    "CH1R":1.36E-3,
    "CH2R":1.36E-3, 
    "CH0":1.837E-3, 
    "CH1":1.933E-3,
    "CH2":2.1775964E-3, 
    "CH3":2.455782E-3, 
    "HO":0.,
    "OR":0.685E-3,
    "OA":1.125E-3, 
    "OE":1.125E-3,
    "NT":1.301E-3,      # 56Acarbo_cht
    "NL":1.301E-3,	# 56Acarbo_cht
    "N":1.301E-3	# 56Acarbo_cht
    }
    
    nvW6 = {
    "HO1O5":0.,
    "HO1C3":0.,
    "HO1C5":0.,
    "HO2C4":0.,
    "HO2O5":0.,
    "HO3C1":0.,
    "HO3C5":0., 
    "HO4C2":0., 
    "HO4O5":0., 
    "C6C1":5.689469E-3,
    "C6C3":5.689469E-3, 
    "O1C3":3.268799E-3,
    "O1C5":3.268799E-3,
    "O2C4":3.268799E-3,
    "O3C1":3.268799E-3,
    "O3C5":3.268799E-3,
    "O4C2":3.268799E-3,
    "O1C6":4.110135E-3,
    "O3C6":4.110135E-3,
    "O1C2":3.268799E-3,
    "H21C4":0.,            # 56Acarbo_cht
    "H21O5":0.,            # 56Acarbo_cht
    "H22C4":0.,            # 56Acarbo_cht
    "H22O5":0.,            # 56Acarbo_cht
    "H23C4":0.,            # 56Acarbo_cht
    "H23O5":0.,            # 56Acarbo_cht
    "N2C4":3.392513E-3     # 56Acarbo_cht
    }
    nvW12 = {
    "HO1O5":0.7E-6,
    "HO1C3":0.35E-6,
    "HO1C5":0.35E-6,
    "HO2C4":0.35E-6,
    "HO2O5":0.35E-6,
    "HO3C1":0.35E-6,
    "HO3C5":0.35E-6, 
    "HO4C2":0.35E-6, 
    "HO4O5":0.35E-6, 
    "C6C1":3.33986E-6,
    "C6C3":3.33986E-6, 
    "O1C3":2.5E-6,
    "O1C5":2.5E-6,
    "O2C4":2.5E-6,
    "O3C1":2.5E-6,
    "O3C5":2.5E-6,
    "O4C2":2.5E-6,
    "O1C6":8.4108E-6,
    "O3C6":8.4108E-6,
    "O1C2":2.5E-6,
    "H21C4":0.35E-6,      # 56Acarbo_cht
    "H21O5":0.35E-6,      # 56Acarbo_cht
    "H22C4":0.35E-6,      # 56Acarbo_cht
    "H22O5":0.35E-6,      # 56Acarbo_cht
    "H23C4":0.35E-6,      # 56Acarbo_cht
    "H23O5":0.35E-6,      # 56Acarbo_cht
    "N2C4":2.5E-6         # 56Acarbo_cht
    } 
    atom1_name = atoms[int(pair1)-1].split()[4]
    atom2_name = atoms[int(pair2)-1].split()[4]
    atom1_type = atoms[int(pair1)-1].split()[1]
    atom2_type = atoms[int(pair2)-1].split()[1]


# 1. O1C2 between rings
    if ( (atom1_name+atom2_name =='O1C2' and atnr2resNr(pair1,atoms)+1 ==atnr2resNr(pair2,atoms)) ):
        array.append('%+5s %+5s   1   %4e %4e; %4s %4s N' % (str(pair1),str(pair2),nvW6[atom1_name+atom2_name],(nvW12[atom1_name+atom2_name]), str(atom1_name), str(atom2_name) ))
    else:
        if ( nvW6.__contains__(atom1_name+atom2_name) and atnr2resNr(pair1,atoms)==atnr2resNr(pair2,atoms) ):
            array.append('%+5s %+5s   1   %4e %4e; %4s %4s N' % (str(pair1),str(pair2),nvW6[atom1_name+atom2_name],
(nvW12[atom1_name+atom2_name]), str(atom1_name), str(atom2_name) ))
        else:
            if (nvW6.__contains__(atom2_name+atom1_name) and atnr2resNr(pair1,atoms)==atnr2resNr(pair2,atoms) ):
                array.append('%+5s %+5s   1   %4e %4e; %4s %4s N' % (str(pair1),str(pair2),nvW6[atom2_name+atom1_name],
(nvW12[atom2_name+atom1_name]), str(atom2_name), str(atom1_name) ))
            else:
                if (atom1_name=='C' or atom2_name=='C' or atom1_name=='O' or atom2_name=='O' or atom1_name=='CH3' or atom2_name=='CH3' ):
                    array.append('%+5s %+5s   1  ' % (str(pair1),str(pair2)))
                else:
                    array.append('%+5s %+5s   1   %4e %4e; %4s %4s' % (str(pair1),str(pair2),vW6[atom1_type]*vW6[atom2_type],vW12[atom1_type]*vW12[atom2_type], str(atom1_name), str(atom2_name)))
    
    
    return array

def atnr2name(nr,atoms):
#returns atom name and atom type for nr in atoms array
    atom_name=atoms[int(nr)-1].split()[4]
    atom_type=atoms[int(nr)-1].split()[1]
    return atom_name, atom_type


def array_remove(array,exclude):
    newlist = [item for item in array if not item.strip().startswith(exclude)]
    return newlist


lines = []
f = open('topol.top','r')

for line in f:
    lines.append(line)
f.close

atoms_index = search_array(lines,'[ atoms ]')
bonds_index = search_array(lines,'[ bonds ]')
pairs_index = search_array(lines,'[ pairs ]')
exclus_index = search_array(lines,'[ exclusions ]')
angles_index = search_array(lines,'[ angles ]')


head = lines[:(pairs_index+1)]
atoms0 = lines[(atoms_index+1):(bonds_index-1)]
pairs0 = lines[(pairs_index+1):(exclus_index-1)]
exclus0 = lines[(exclus_index+1):(angles_index-1)]
tail = lines[exclus_index:]

atoms = array_remove(atoms0,";") 
pairs = array_remove(pairs0,";")
exclus = array_remove(exclus0,";")

pairs_new = []
for index,it in enumerate(pairs):
# LJ interactions for [ pairs ] section
    item = it.strip()
    pair1 = item.split()[0]
    pair2 = item.split()[1]
    inter(pair1,pair2,atoms,pairs_new)

for index,it in enumerate(exclus):
# LJ interactions for [ exclusions ] section
    item = it.strip()
    if (len(item.split()) == 2):
      pair1 = item.split()[0]
      pair2 = item.split()[1]
      inter(pair1,pair2,atoms,pairs_new)
    if (len(item.split()) == 3):
     pair1=item.split()[0]     
     pair2=item.split()[1]
     pair3=item.split()[2]
     inter(pair1,pair2,atoms,pairs_new)
     inter(pair1,pair3,atoms,pairs_new)  

#for index,it in enumerate(atoms):
#    item=it.strip()
#    print item

top_new = open('top_new.top','w')

for i,item in enumerate(head):
     top_new.write((item))

for item in pairs_new:
     top_new.write((item+"\n"))

top_new.write("\n")


for item in tail:
     top_new.write((item))

